
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * A first-in, first-out bounded collection of Message objects.
 */
public class BoundedQueue 
{
    private Message elements[];
    private int head;
    private int tail;
    private int size;
    
    private Lock qLock = new ReentrantLock();

    /**
     * Constructs an empty queue.
     * @param capacity the maximum capacity of the queue
     */
    public BoundedQueue(int capacity) 
    {
        elements = new Message[capacity];
        head = 0;
        tail = 0;
        size = 0;
    }

    /**
     * Removes the object at the head.
     * @return the object that has been removed from the queue 
     * @precondition !isEmpty()
     */
    public Message remove() 
    {
        qLock.lock();
        
        try {
            Message msg = elements[head];
            
            head++;
            size--;
            if (head == elements.length) head = 0;

            return msg;
        }
        finally {
            qLock.unlock();
        }
    }

    /**
     * Appends an object at the tail.
     * @param newValue the object to be appended 
     * @precondition !isFull();
     */
    public void add(Message newValue) 
    {
        qLock.lock();
        
        try {
            elements[tail] = newValue;
            
            tail++;
            size++;
            if (tail == elements.length) tail = 0;
        }
        finally {
            qLock.unlock();
        }
    }
    
    public int getSize() { return size; }

    public boolean isFull() 
    {
        return size == elements.length;
    }

    public boolean isEmpty() 
    {
        return size == 0;
    }
}
